@@ -18,5 +18,5 @@
 .onLoad <- function(libname, pkgname) {
 	where <- parent.env(environment())
 	fitters <- c("linear","nnetTs","setar","lstar","star","aar")
-	assign("fitters", fitters, env=where)	
+	assign("fitters", fitters, envir=where)	
 }
