@@ -34,7 +34,7 @@ data(Canada)
 
 myVECM<-VECM(Canada, lag=1, include="const", estim="ML")
 VECM_vars<-cajorls(ca.jo(Canada, spec="trans"))
-all.equal(VECM_vars$beta, myVECM$model.specific$coint, check.attributes=FALSE)
+print(all.equal(VECM_vars$beta, myVECM$model.specific$coint, check.attributes=FALSE))
 
 
 ## Check LL
@@ -42,16 +42,16 @@ l1<-2*(logLik(myVECM,r=4)-logLik(myVECM,r=3))
 l2<-2*(logLik(myVECM,r=3)-logLik(myVECM,r=2))
 l3<-2*(logLik(myVECM,r=2)-logLik(myVECM,r=1))
 l4<-2*(logLik(myVECM,r=1)-logLik(myVECM,r=0))
-l1;l2;l3;l4
-ca.jo(Canada, spec="trans")
-all.equal(c(l1, l2, l3, l4),ca.jo(Canada, spec="trans")@teststat)
+print(c(l1,l2,l3,l4))
 logLik(myVECM,r=5)
+print(ca.jo(Canada, spec="trans"))
+print(all.equal(c(l1, l2, l3, l4),ca.jo(Canada, spec="trans")@teststat))
 
-AIC(myVECM,r=0, k=2*log(log(myVECM$t)))
-AIC(myVECM,r=1, k=2*log(log(myVECM$t)))
-AIC(myVECM,r=2, k=2*log(log(myVECM$t)))
-AIC(myVECM,r=3, k=2*log(log(myVECM$t)))
-AIC(myVECM,r=4, k=2*log(log(myVECM$t)))
+print(AIC(myVECM,r=0, k=2*log(log(myVECM$t))))
+print(AIC(myVECM,r=1, k=2*log(log(myVECM$t))))
+print(AIC(myVECM,r=2, k=2*log(log(myVECM$t))))
+print(AIC(myVECM,r=3, k=2*log(log(myVECM$t))))
+print(AIC(myVECM,r=4, k=2*log(log(myVECM$t))))
 
 
 }
